local E, F, C = unpack(select(2, ...))
local type, gsub, time, floor,  _ = type, _G.string.gsub, time, math.floor, _

ChatFontNormal:SetFont(C.CHAT_FONT, 13)
ChatFontNormal:SetShadowOffset(1, -1)
ChatFontNormal:SetShadowColor(0, 0, 0, 1)
ChatFontNormal:SetSpacing(3)

DEFAULT_CHATFRAME_ALPHA = 0
CHAT_FRAME_TAB_SELECTED_MOUSEOVER_ALPHA = 1
CHAT_FRAME_TAB_SELECTED_NOMOUSE_ALPHA = 1 
CHAT_FRAME_TAB_ALERTING_MOUSEOVER_ALPHA = 1
CHAT_FRAME_TAB_ALERTING_NOMOUSE_ALPHA = 1 
CHAT_FRAME_TAB_NORMAL_MOUSEOVER_ALPHA = 1
CHAT_FRAME_TAB_NORMAL_NOMOUSE_ALPHA = 1 
CHAT_FRAME_FADE_OUT_TIME = .1
CHAT_FRAME_FADE_TIME = .1
CHAT_TAB_SHOW_DELAY = 0
CHAT_TAB_HIDE_DELAY = 0

CHAT_FONT_HEIGHTS = {
	[1] = 8, [2] = 9, [3] = 10, [4] = 11, [5] = 12, [6] = 13, [7] = 14, [8] = 15, [9] = 16, [10] = 17, [11] = 18, [12] = 19, [13] = 20
}

CHAT_FLAG_AFK = 'AFK - '
CHAT_FLAG_DND = 'DND - '
CHAT_FLAG_GM = 'GM - '

CHAT_SAY_GET = "%s: "
CHAT_YELL_GET = "%s: "

CHAT_GUILD_GET = '|Hchannel:Guild|h/g|h %s:\32'
CHAT_OFFICER_GET = '|Hchannel:o|h/o|h %s:\32'

CHAT_RAID_GET = '|Hchannel:raid|h/r|h %s:\32'
CHAT_RAID_WARNING_GET = '[RW!] %s:\32'
CHAT_RAID_LEADER_GET = '|Hchannel:raid|h/r|h %s:\32'

CHAT_BATTLEGROUND_GET = '|Hchannel:Battleground|h/bg|h %s:\32'
CHAT_BATTLEGROUND_LEADER_GET = '|Hchannel:Battleground|h/bg|h %s:\32'

CHAT_INSTANCE_CHAT_GET = "|Hchannel:INSTANCE_CHAT|h/i|h %s:\32"
CHAT_INSTANCE_CHAT_LEADER_GET = "|Hchannel:INSTANCE_CHAT|h/i|h %s:\32"

CHAT_PARTY_GET = '|Hchannel:party|h/p|h %s:\32'
CHAT_PARTY_LEADER_GET = '|Hchannel:party|h/p|h %s:\32'
CHAT_PARTY_GUIDE_GET = '|Hchannel:party|h/p|h %s:\32'
CHAT_MONSTER_PARTY_GET = '|Hchannel:raid|h/r|h %s:\32'

_G.GOLD_AMOUNT = '|cffffffff%d|r|TInterface\\MONEYFRAME\\UI-GoldIcon:14:14:2:0|t'
_G.SILVER_AMOUNT = '|cffffffff%d|r|TInterface\\MONEYFRAME\\UI-SilverIcon:14:14:2:0|t'
_G.COPPER_AMOUNT = '|cffffffff%d|r|TInterface\\MONEYFRAME\\UI-CopperIcon:14:14:2:0|t'

local _, class = UnitClass('player')
local color = CUSTOM_CLASS_COLORS and CUSTOM_CLASS_COLORS[class] or RAID_CLASS_COLORS[class]
local AddMessage = ChatFrame1.AddMessage

hooksecurefunc('FloatingChatFrame_OnMouseScroll', function(self, direction)
	if(direction > 0) then
		if(IsShiftKeyDown()) then
			self:ScrollToTop()
		else
			self:ScrollUp()
		end
	else
		if(IsShiftKeyDown()) then
			self:ScrollToBottom()
		else
			self:ScrollDown()
		end
	end
end)

FCFTab_UpdateColors = function(self, selected)
	if (selected) then
		self:GetFontString():SetTextColor(color.r, color.g, color.b)
		if self.TabTex then
			self.TabTex:SetAlpha(.65)
		end
	else
		self:GetFontString():SetTextColor(.5, .5, .5)
		if self.TabTex then
			self.TabTex:SetAlpha(.25)
		end
	end
end

local FCF_AddMessage = function(self, text, ...)
	if type(text) == 'string' then
		local _, size = self:GetFont()
		local iconsize = 10
		size = floor(size + .5)

		if text:find' |T' then                                    -- sharpen & resize icons
			text = gsub(text, '(:12:12)', ':'..iconsize..':'..iconsize..':0:0:64:64:5:59:5:59')
		end

		text = gsub(text, "(|HBNplayer.-|h)%[(.-)%]|h", "%1%2|h")	-- battlenet player name
		text = gsub(text, "(|Hplayer.-|h)%[(.-)%]|h", "%1%2|h")		-- player name
		text = gsub(text, "%[(%d0?)%. (.-)%]", "/%1")				-- globals channels: [1. trade] > /1./
		text = gsub(text, "Guild Message of the Day:", "GMotD -")	-- message of the day
		text = gsub(text, "To (|HBNplayer.+|h):", "%1 >>")			-- whisper to bnet
		text = gsub(text, "To (|Hplayer.+|h):", "%1 >>")				-- whisper to
		text = gsub(text, ' whispers:', ' <<')						-- whisper from
		text = gsub(text, 'Joined Channel:', '+')					-- channel join
		text = gsub(text, 'Left Channel:', '- ')					-- channel left
		text = gsub(text, 'Changed Channel:', '>')					-- channel change
	end
	return AddMessage(self, text, ...)
end

local UpdateChatTabStyle = function(self)
	local chat = _G[self]

	-- hide textures
	local tab = _G[self..'Tab']
	for i = 1, select('#', tab:GetRegions()) do
		local texture = select(i, tab:GetRegions())
		if texture and texture:GetObjectType() == 'Texture' then
			texture:SetTexture(nil)
		end
	end
	
	local tabtex = tab:CreateTexture(nil, 'BACKGROUND')
	tabtex:SetTexture[[Interface\QuestFrame\AutoQuest-Parts]]
	tabtex:SetTexCoord(.42, .96, 1, 0)
	tabtex:SetAllPoints(tab)
	tabtex:SetHeight(43)
	tabtex:SetAlpha(.65)	
	tab.TabTex = tabtex

	-- text
	local tabText = _G[self..'TabText']
	tabText:SetFont(C.BIG_FONT, 10)
	tabText:SetShadowOffset(.7, -.7)
	tabText:SetShadowColor(0, 0, 0, 1)
	tabText:SetDrawLayer('OVERLAY', 7)
	tabText:SetTextColor(color.r, color.g, color.b)
	tabText:SetAlpha(0)

	-- move tab text
	local a = {tabText:GetPoint()}
	tabText:SetPoint(a[1], a[2], a[3], a[4], -3) --2)

	chat.tabStyled = true
end

local UpdateChatFrameStyle = function(self)
	local chat = _G[self]

	-- boundaries
	chat:SetShadowOffset(1, -1)
	chat:SetClampedToScreen(false)
	chat:SetClampRectInsets(0, 0, 0, 0)
	chat:SetMaxResize(UIParent:GetWidth(), UIParent:GetHeight())
	chat:SetMinResize(150, 25)
	chat:SetSpacing(3)
	chat:SetShadowOffset(1, -1)
	chat:SetShadowColor(0, 0, 0, 1)

	-- hide the weird extra background for now non-existent buttons
	for _, texture in pairs({
		'ButtonFrameBackground',
		'ButtonFrameTopLeftTexture',
		'ButtonFrameBottomLeftTexture',
		'ButtonFrameTopRightTexture',
		'ButtonFrameBottomRightTexture',
		'ButtonFrameLeftTexture',
		'ButtonFrameRightTexture',
		'ButtonFrameBottomTexture',
		'ButtonFrameTopTexture',
	}) do
		if _G[self..texture] then
			_G[self..texture]:SetTexture(nil)
		end
	end

	-- edit box
	local editbox = _G[self..'EditBox']
	local header = _G[self..'EditBoxHeader']
	local suffix = _G[self..'EditBoxHeaderSuffix']
		
	_G[self..'EditBoxLeft']:Hide()
	_G[self..'EditBoxMid']:Hide()
	_G[self..'EditBoxRight']:Hide()
	
	editbox:SetAltArrowKeyMode(false)
	editbox:ClearAllPoints()
	editbox:SetPoint('TOPLEFT', self, 'BOTTOMLEFT', -2, -10)
	editbox:SetPoint('TOPRIGHT', self, 'BOTTOMRIGHT', 2, -10)
	
	F:CreateBackground(editbox, 0.8, C.TOOLTIP_BACKGROUND_INSET)
	F:CreateBorder(editbox, C.TOOLTIP_BORDER_INSET)
	
	editbox:SetHeight(24)
	editbox:SetTextInsets(12 + header:GetWidth() + (suffix:IsShown() and suffix:GetWidth() or 0), 12, 0, 0)
	editbox:SetBorderColor(unpack(C.BORDER_COLOR_DARK))

	editbox:SetFont(C.CHAT_FONT, 13)
	header:SetFont(C.CHAT_FONT, 13)
	suffix:SetFont(C.CHAT_FONT, 13)

	chat.isStyled = true
end

local UpdateChat = function()
	-- make chat windows transparent
	for i = 1, NUM_CHAT_WINDOWS do
		SetChatWindowAlpha(i, 0)
		
		if ( i ~= 2 ) then
			local f = _G["ChatFrame"..i]
			f.AddMessage = FCF_AddMessage
		end
	end

	-- style
	for _, v in pairs(CHAT_FRAMES) do
		local chat = _G[v]
		if chat and not chat.isStyled then
			UpdateChatFrameStyle(chat:GetName())
		end
	end
end

local UpdateTab = function(self, chat)
	for i, v in ipairs(GENERAL_CHAT_DOCK.DOCKED_CHAT_FRAMES) do
		if not v.tabStyled then
			UpdateChatTabStyle(v:GetName())
		end
	end
end

hooksecurefunc('FCFDock_UpdateTabs',  UpdateTab)
hooksecurefunc('FCF_OpenTemporaryWindow', UpdateChat)
UpdateChat()