local E, F, C = unpack(select(2, ...))

local WIDTH 		= 200
local HEIGHT 		= 8
local _, CLASS 		= UnitClass('player')
local COLOR			= CUSTOM_CLASS_COLORS and CUSTOM_CLASS_COLORS[CLASS] or RAID_CLASS_COLORS[CLASS]
local POSITION		= {'TOP', Minimap, 'BOTTOM', 0, -20 }
local OFFSET		= -15
local TIP			= {'TOPRIGHT', UIParent, -275, -235}
local TEXTURE 		= C.STATUSBAR_TEXTURE

local fr = CreateFrame('Frame', nil, UIParent)
fr:SetPoint(POSITION[1], POSITION[2], POSITION[3], POSITION[4], POSITION[5])
fr:SetWidth(WIDTH)
fr:SetHeight(HEIGHT)

-- SETUP BARS
local setBar = function(frame)
	frame:SetStatusBarTexture(TEXTURE)
	frame:SetWidth(WIDTH)
	frame:SetHeight(HEIGHT)
end

local setBackdrop = function(frame)
	F:CreateBorder(frame, -1)
	frame:SetBorderColor(unpack(C.BORDER_COLOR_GOLD))

	frame.bg = CreateFrame('StatusBar', nil, frame)
	frame.bg:SetStatusBarTexture(C.STATUSBAR_TEXTURE)
	frame.bg:SetPoint('TOPLEFT', frame, -1, 1)
	frame.bg:SetPoint('BOTTOMRIGHT', frame, 1, -1)
	frame.bg:SetFrameLevel(0)
	frame.bg:SetStatusBarColor(.1, .1, .1, 1)
end

local exp = CreateFrame('StatusBar', nil, fr, 'AnimatedStatusBarTemplate')
setBar(exp)
exp:SetFrameLevel(2)
exp:SetStatusBarColor(.4, .1, .6)
exp:SetAnimatedTextureColors(.4, .1, .6)
exp:SetPoint(POSITION[1], POSITION[2], POSITION[3], POSITION[4], POSITION[5])
setBackdrop(exp)

local rest = CreateFrame('StatusBar', nil, xp)
setBar(rest)
rest:SetFrameLevel(1)
rest:EnableMouse(false)
rest:SetStatusBarColor(.2, .4, .8)
rest:SetAllPoints(exp)


-- DATA METHODS
local factionStanding = {
	[1] = { name = 'Hated' },
	[2] = { name = 'Hostile' },
	[3] = { name = 'Unfriendly' },
	[4] = { name = 'Neutral' },
	[5] = { name = 'Friendly' },
	[6] = { name = 'Honored' },
	[7] = { name = 'Revered' },
	[8] = { name = 'Exalted' },
};

local numberize = function(v)
    if v <= 9999 then return v end
    if v >= 1000000 then
        local value = string.format('%.2fm', v/1000000)
        return value
    elseif v >= 10000 then
        local value = string.format('%.2fk', v/1000)
        return value
    end
end

local exp_update = function()
	if UnitLevel('player') == MAX_PLAYER_LEVEL then
		local name, standing, min, max, cur, factionID = GetWatchedFactionInfo()
		if name then
			local faction = FACTION_BAR_COLORS[standing]
		
			exp:SetStatusBarColor(faction.r, faction.g, faction.b)
			exp:SetAnimatedTextureColors(faction.r, faction.g, faction.b)
			exp:SetAnimatedValues(cur - min, 0, max - min, standing)

			rest:SetMinMaxValues(0, 1)
			rest:SetValue(0)
			
			exp:Show()
			rest:Show()
			return
		end

		exp:Hide()
		rest:Hide()
	else
		local c, m, l	= UnitXP('player'), UnitXPMax('player'), UnitLevel('player')
		local r			= GetXPExhaustion()
		local p = 0

		if c > 0 and m > 0 then
			p = math.ceil(c/m*100)
		end

		exp:SetAnimatedValues(c, 0, m, l)
		rest:SetMinMaxValues(min(0, c), m)
		rest:SetValue(r and (c + r) or 0)

		exp:Show()
		rest:Show()
	end
end

local showExperienceTooltip = function(self)
	if UnitLevel('player') == MAX_PLAYER_LEVEL then
		local name, standing, min, max, cur, factionID = GetWatchedFactionInfo()
		if name then
			local faction = FACTION_BAR_COLORS[standing]
			
			GameTooltip:SetOwner(self, 'ANCHOR_NONE')
			GameTooltip:SetPoint(TIP[1], TIP[2], TIP[3], TIP[4], TIP[5])
			GameTooltip:AddLine(name, faction.r, faction.g, faction.b)
						
			GameTooltip:AddLine(factionStanding[standing].name, 1, 1, 1)
			GameTooltip:AddLine(cur-min.."/"..max-min, 1, 1, 1)
			
			GameTooltip:Show()
		end
	else
		local xpc, xpm, xpr = UnitXP('player'), UnitXPMax('player'), GetXPExhaustion('player')

		GameTooltip:SetOwner(self, 'ANCHOR_NONE')
		GameTooltip:SetPoint(TIP[1], TIP[2], TIP[3], TIP[4], TIP[5])

		GameTooltip:AddLine('Level '..UnitLevel('player'), COLOR.r, COLOR.g, COLOR.b)
		GameTooltip:AddLine((numberize(xpc)..'/'..numberize(xpm)..' ('..floor((xpc/xpm)*100) ..'%)'), 1, 1, 1)
		if xpr then
			GameTooltip:AddLine(numberize(xpr)..' ('..floor((xpr/xpm)*100) ..'%)', .2, .4, .8)
		end

		GameTooltip:Show()
	end
end

-- events
exp:RegisterEvent('PLAYER_LEVEL_UP')
exp:RegisterEvent('PLAYER_XP_UPDATE')
exp:RegisterEvent('UPDATE_EXHAUSTION')
exp:RegisterEvent('PLAYER_ENTERING_WORLD')
exp:RegisterEvent('MODIFIER_STATE_CHANGED')
exp:RegisterEvent('PLAYER_UPDATE_RESTING')
exp:RegisterEvent('UPDATE_FACTION')
exp:RegisterEvent('ADDON_LOADED')
exp:SetScript('OnEvent', exp_update)
exp:SetScript('OnEnter', function() showExperienceTooltip(exp) end)
exp:SetScript('OnLeave', function() GameTooltip:Hide() end)

hooksecurefunc("SetWatchedFactionIndex", function(self)
	exp_update()
end)

-- resting indicator
local indicator = CreateFrame('Frame', nil, fr)
indicator:SetPoint('LEFT', fr, 5, 5)
indicator:SetSize(20, 10)
indicator:SetFrameLevel(5)
indicator:EnableMouse(false)

t = indicator:CreateFontString(nil, 'BACKGROUND')
t:SetPoint('LEFT', indicator)
t:SetFont(C.SMALL_FONT, 10, 'OUTLINE')
t:SetShadowOffset(1, -1)
t:SetShadowColor(0, 0, 0)
t:SetJustifyH('LEFT')
t:SetText('|cff00DDFFzZz|r')

indicator:RegisterEvent('PLAYER_ENTERING_WORLD')
indicator:RegisterEvent('PLAYER_UPDATE_RESTING')

indicator:SetScript('OnUpdate', function(self, event, ...)
	if IsResting() then
		self:SetAlpha(1)
	else
		self:SetAlpha(0)
	end
end)