local E, F, C = unpack(select(2, ...))

local sections = {"TOPLEFT", "TOPRIGHT", "BOTTOMLEFT", "BOTTOMRIGHT", "TOP", "BOTTOM", "LEFT", "RIGHT"}

local SetBorderColor = function(self, r, g, b, a)
	local t = self.borderTextures
	if not t then return end

	for _, tex in pairs(t) do
		tex:SetVertexColor(r or 1, g or 1, b or 1, a or 1)
	end
end

local GetBorderColor = function(self)
	return self.borderTextures and self.borderTextures.TOPLEFT:GetVertexColor()
end

function F:CreateBorder(object, offset)
	if type(object) ~= "table" or not object.CreateTexture or object.borderTextures then return end

	local t = {}
	offset = offset or 0

	for i = 1, #sections do
		local x = object:CreateTexture(nil, "OVERLAY", nil, 1)
		x:SetTexture("Interface\\AddOns\\lyn\\assets\\border\\"..sections[i], i > 4 and true or nil)
		t[sections[i]] = x
	end

	t.TOPLEFT:SetSize(8, 8)
	t.TOPLEFT:SetPoint("BOTTOMRIGHT", object, "TOPLEFT", 6 + offset, -6 - offset)

	t.TOPRIGHT:SetSize(8, 8)
	t.TOPRIGHT:SetPoint("BOTTOMLEFT", object, "TOPRIGHT", -6 - offset, -6 - offset)

	t.BOTTOMLEFT:SetSize(8, 8)
	t.BOTTOMLEFT:SetPoint("TOPRIGHT", object, "BOTTOMLEFT", 6 + offset, 6 + offset)

	t.BOTTOMRIGHT:SetSize(8, 8)
	t.BOTTOMRIGHT:SetPoint("TOPLEFT", object, "BOTTOMRIGHT", -6 - offset, 6 + offset)

	t.TOP:SetHeight(8)
	t.TOP:SetHorizTile(true)
	t.TOP:SetPoint("TOPLEFT", t.TOPLEFT, "TOPRIGHT", 0, 2)
	t.TOP:SetPoint("TOPRIGHT", t.TOPRIGHT, "TOPLEFT", 0, 2)

	t.BOTTOM:SetHeight(8)
	t.BOTTOM:SetHorizTile(true)
	t.BOTTOM:SetPoint("BOTTOMLEFT", t.BOTTOMLEFT, "BOTTOMRIGHT", 0, -2)
	t.BOTTOM:SetPoint("BOTTOMRIGHT", t.BOTTOMRIGHT, "BOTTOMLEFT", 0, -2)

	t.LEFT:SetWidth(8)
	t.LEFT:SetVertTile(true)
	t.LEFT:SetPoint("TOPLEFT", t.TOPLEFT, "BOTTOMLEFT", -2, 0)
	t.LEFT:SetPoint("BOTTOMLEFT", t.BOTTOMLEFT, "TOPLEFT", -2, 0)

	t.RIGHT:SetWidth(8)
	t.RIGHT:SetVertTile(true)
	t.RIGHT:SetPoint("TOPRIGHT", t.TOPRIGHT, "BOTTOMRIGHT", 2, 0)
	t.RIGHT:SetPoint("BOTTOMRIGHT", t.BOTTOMRIGHT, "TOPRIGHT", 2, 0)

	object.borderTextures = t
	object.SetBorderColor = SetBorderColor
	object.GetBorderColor = GetBorderColor
end

function F:CreateBackground(frame, alpha, inset)
	if not frame then return end
	local f = frame
	if f:GetObjectType() == 'Texture' then
		if not frame.Background then
			local parent = frame:GetParent()
			frame.Background = CreateFrame('Frame', nil, parent)
			frame.Background:SetAllPoints(frame)
			frame.Background:SetFrameLevel(parent:GetFrameLevel() - 1)
		end
		f = frame.Background
	end
	f:SetBackdrop({
		bgFile = [[Interface/ChatFrame/ChatFrameBackground]],
		tiled = false,
		insets = {
			left 	= inset or -3,
			right 	= inset or -3,
			top		= inset or -3,
			bottom	= inset or -3
		}
	})
	f:SetBackdropColor(0, 0, 0, alpha or 1)
end

function F:CreateStatusbarTexture(frame)
	if frame:GetObjectType() == 'StatusBar' then
		frame:SetStatusBarTexture(C.STATUSBAR_TEXTURE)
	else
		frame:SetTexture(C.STATUSBAR_TEXTURE)
	end
end