local E, F, C = unpack(select(2, ...))

local _, class = UnitClass'player'
local color = CUSTOM_CLASS_COLORS and CUSTOM_CLASS_COLORS[class] or RAID_CLASS_COLORS[class]
local font  = C.BIG_FONT
local width = 200
 
-- square
MinimapCluster:ClearAllPoints()
MinimapCluster:SetPoint('TOPRIGHT', UIParent, 'TOPRIGHT', -50, -50)
MinimapCluster:EnableMouse(false)
MinimapCluster:SetClampedToScreen(false)
MinimapCluster:SetSize(width, width)
Minimap:SetClampedToScreen(false)
Minimap:SetSize(width, width)
Minimap:SetMaskTexture([[Interface/Buttons/WHITE8X8]])
Minimap:ClearAllPoints()
Minimap:SetAllPoints(MinimapCluster)
Minimap:SetFrameLevel(2)

function GetMinimapShape() return 'SQUARE' end

-- border
local Border = Minimap:CreateTexture(nil, 'OVERLAY', nil, 2)
Border:SetTexture([[Interface\AddOns\lyn\assets\minimap.tga]])
Border:SetPoint('TOPLEFT', -15, 15)
Border:SetPoint('BOTTOMRIGHT', 15, -15)
Border:SetVertexColor(unpack(C.BORDER_COLOR_GOLD))

-- menu
Minimap:SetScript('OnMouseUp', function(self, button)
	Minimap:StopMovingOrSizing()
	if button == 'RightButton' then
		ToggleDropDownMenu(1, nil, MiniMapTrackingDropDown, self, - (Minimap:GetWidth() * .005), 5)
	else
		Minimap_OnClick(self)
	end
end)

-- zoom
Minimap:EnableMouseWheel(true)
Minimap:SetScript('OnMouseWheel', function(self, arg1)
	if arg1 > 0 then Minimap_ZoomIn() else Minimap_ZoomOut() end
end)

-- hide
for _, v in pairs({
	MinimapBackdrop,
	MinimapBorder,
	MinimapBorderTop,
	MinimapToggleButton,
	MinimapZoomOut,
	MinimapZoomIn,
	MiniMapWorldMapButton,
	GameTimeFrame,
	MinimapZoneTextButton,
	MiniMapTracking,
}) do
	if v then v:Hide() end
end

for _, v in pairs({
	MinimapNorthTag,
}) do
	if v then v:SetAlpha(0) end
end

DurabilityFrame:ClearAllPoints()
DurabilityFrame:SetParent(Minimap)
DurabilityFrame:SetFrameStrata'HIGH'
DurabilityFrame:SetPoint('BOTTOMRIGHT', UIParent, -50, 100)
DurabilityFrame.SetPoint = function() end

-- mail
MiniMapMailFrame:ClearAllPoints()
MiniMapMailFrame:SetParent(Minimap)
MiniMapMailFrame:SetFrameStrata'HIGH'
MiniMapMailFrame:SetPoint('BOTTOMRIGHT', Minimap, -13, -18)
MiniMapMailBorder:SetVertexColor(unpack(C.BORDER_COLOR_GOLD))

-- difficulty
--MiniMapInstanceDifficulty:ClearAllPoints()
--MiniMapInstanceDifficulty:SetPoint('TOPLEFT', Minimap, 10, 7)
--GuildInstanceDifficulty:ClearAllPoints()
--GuildInstanceDifficulty:SetPoint('TOPLEFT', Minimap, 10, 7)
--MiniMapChallengeMode:ClearAllPoints()
--MiniMapChallengeMode:SetPoint('TOPLEFT', Minimap, 10, 7)

-- lfg
local lfg = MiniMapLFGFrame or QueueStatusMinimapButton
if lfg then
	lfg:SetScale(.9)
	lfg:ClearAllPoints()
	lfg:SetParent(Minimap)
	lfg:SetFrameStrata'HIGH'
	lfg:SetPoint('BOTTOMLEFT', Minimap, 15, -18)
	QueueStatusMinimapButtonBorder:SetVertexColor(unpack(C.BORDER_COLOR_GOLD))
end

-- world capture bar
hooksecurefunc('UIParent_ManageFramePositions', function()
	if NUM_EXTENDED_UI_FRAMES then
		for i = 1, NUM_EXTENDED_UI_FRAMES do
			local bar = _G['WorldStateCaptureBar'..i]
			if bar and bar:IsVisible() then
				bar:ClearAllPoints()
				if i == 1 then
					bar:SetPoint('BOTTOM', MinimapCluster, 'TOP', 0, 0)
				else
					bar:SetPoint('BOTTOM', _G['WorldStateCaptureBar'..(i - 1)], 'TOP', 0, 20)
				end
			end
		end
	end
end)

-- zone text
local function GetZoneColor()
	local zoneType = GetZonePVPInfo()
	if (zoneType == 'sanctuary') then
		return 0.4, 0.8, 0.94
	elseif (zoneType == 'arena') then
		return 1, 0.1, 0.1
	elseif (zoneType == 'friendly') then
		return 0.1, 1, 0.1
	elseif (zoneType == 'hostile') then
		return 1, 0.1, 0.1
	elseif (zoneType == 'contested' or zoneType == 'combat') then
		return 1, 0.8, 0
	else
		return 1, 1, 1
	end
end

local zone = Minimap:CreateFontString(nil, 'OVERLAY')
zone:SetFont(font, 12, 'OUTLINE')
zone:SetPoint('TOPRIGHT', Minimap, 2, 20)
zone:SetShadowOffset(1, -1)
zone:SetShadowColor(0, 0, 0)
zone:SetAlpha(1)
zone:SetSize(width, 10)
zone:SetJustifyH('RIGHT')
zone:SetTextColor(GetZoneColor())
if GetSubZoneText() == '' then
	zone:SetText(string.upper(GetZoneText()))
else
	zone:SetText(string.upper(GetSubZoneText()))
end

Minimap:HookScript('OnUpdate', function()
	if GetSubZoneText() == '' then
		zone:SetText(string.upper(GetZoneText()))
	else
		zone:SetText(string.upper(GetSubZoneText()))
	end
	zone:SetTextColor(GetZoneColor())
end)


