local E, F, C = unpack(select(2, ...))

GAME_TOOLTIP_BACKDROP_STYLE_DEFAULT['edgeFile'] = ''
GAME_TOOLTIP_BACKDROP_STYLE_DEFAULT['insets'] 	= {left = 0, right = 0, top = 0, bottom = 0}

TOOLTIP_DEFAULT_COLOR.r = 0
TOOLTIP_DEFAULT_COLOR.g = 0
TOOLTIP_DEFAULT_COLOR.b = 0

TOOLTIP_DEFAULT_BACKGROUND_COLOR.r = 0
TOOLTIP_DEFAULT_BACKGROUND_COLOR.g = 0
TOOLTIP_DEFAULT_BACKGROUND_COLOR.b = 0

GameTooltipHeaderText:SetFont(C.TOOLTIP_FONT, 18)
GameTooltipHeaderText:SetShadowOffset(1, -1)
GameTooltipHeaderText:SetShadowColor(0, 0, 0, 1)

GameTooltipText:SetFont(STANDARD_TEXT_FONT, 12)
GameTooltipText:SetShadowOffset(1, -1)
GameTooltipText:SetShadowColor(0, 0, 0,1)

Tooltip_Small:SetFont(STANDARD_TEXT_FONT, 12)
Tooltip_Small:SetShadowOffset(1, -1)
Tooltip_Small:SetShadowColor(0, 0, 0, 1)

SmallTextTooltipText:SetFont(STANDARD_TEXT_FONT, 12)

local classifications = {
	worldboss  = 'Boss ',
	rareelite  = 'Rare Elite ',
	rare       = 'Rare ',
	elite      = 'Elite ',
}

local tooltips = {
	'GameTooltip',
	'ShoppingTooltip1',
	'ShoppingTooltip2',
	'ItemRefTooltip',
	'ItemRefShoppingTooltip1',
	'ItemRefShoppingTooltip2',
	'WorldMapTooltip',
}

local GetHexColor = function(colour)
	return ('%.2x%.2x%.2x'):format(colour.r*255, colour.g*255, colour.b*255)
end

local reactionColors = {}
for i = 1, #FACTION_BAR_COLORS do
	reactionColors[i] = GetHexColor(FACTION_BAR_COLORS[i])
end

local classColors = {}
for class, color in pairs(RAID_CLASS_COLORS) do
	classColors[class] = GetHexColor(CUSTOM_CLASS_COLORS and CUSTOM_CLASS_COLORS[class] or RAID_CLASS_COLORS[class])
end

local ShowCompareItem = function(self, anchor)
	if ( not self ) then
		self = GameTooltip;
	end

	if( not anchor ) then
		anchor = self.overrideComparisonAnchorFrame or self;
	end

	if ( self.needsReset ) then
		self:ResetSecondaryCompareItem();
		GameTooltip_AdvanceSecondaryCompareItem(self);
		self.needsReset = false;
	end
	
	local shoppingTooltip1, shoppingTooltip2 = unpack(self.shoppingTooltips);
	local primaryItemShown, secondaryItemShown = shoppingTooltip1:SetCompareItem(shoppingTooltip2, self);
	
	-- 1
		shoppingTooltip1:SetBackdrop(BACKDROP)
		shoppingTooltip1:SetBackdropColor(TOOLTIP_DEFAULT_BACKGROUND_COLOR.r, TOOLTIP_DEFAULT_BACKGROUND_COLOR.g, TOOLTIP_DEFAULT_BACKGROUND_COLOR.b)

		ShoppingTooltip1TextLeft2:SetFont(C.TOOLTIP_FONT, 14)
		ShoppingTooltip1TextLeft2:SetShadowOffset(1, -1)
		ShoppingTooltip1TextLeft2:SetShadowColor(0, 0, 0, 1)

		ShoppingTooltip1TextLeft3:SetFont(STANDARD_TEXT_FONT, 12)
		ShoppingTooltip1TextLeft3:SetShadowOffset(1, -1)
		ShoppingTooltip1TextLeft3:SetShadowColor(0, 0, 0, 1)

		-- 2
		shoppingTooltip2:SetBackdrop(BACKDROP)
		shoppingTooltip2:SetBackdropColor(TOOLTIP_DEFAULT_BACKGROUND_COLOR.r, TOOLTIP_DEFAULT_BACKGROUND_COLOR.g, TOOLTIP_DEFAULT_BACKGROUND_COLOR.b)

		ShoppingTooltip2TextLeft2:SetFont(C.TOOLTIP_FONT, 14)
		ShoppingTooltip2TextLeft2:SetShadowOffset(1, -1)
		ShoppingTooltip2TextLeft2:SetShadowColor(0, 0, 0, 1)

		ShoppingTooltip2TextLeft3:SetFont(STANDARD_TEXT_FONT, 12)
		ShoppingTooltip2TextLeft3:SetShadowOffset(1, -1)
		ShoppingTooltip2TextLeft3:SetShadowColor(0, 0, 0, 1)

		local leftPos = anchor:GetLeft();
		local rightPos = anchor:GetRight();

		local side;
		if ( self.overrideComparisonAnchorSide ) then
			side = self.overrideComparisonAnchorSide;
		else
			-- find correct side
			local rightDist = 0;
			if ( not rightPos ) then
				rightPos = 0;
			end
			if ( not leftPos ) then
				leftPos = 0;
			end

			rightDist = GetScreenWidth() - rightPos;

			if (leftPos and (rightDist < leftPos)) then
				side = "left";
			else
				side = "right";
			end
		end

		-- see if we should slide the tooltip
		if ( self:GetAnchorType() and self:GetAnchorType() ~= "ANCHOR_PRESERVE" ) then
			local totalWidth = 0;
			if ( primaryItemShown  ) then
				totalWidth = totalWidth + shoppingTooltip1:GetWidth();
			end
			if ( secondaryItemShown  ) then
				totalWidth = totalWidth + shoppingTooltip2:GetWidth();
			end

			if ( (side == "left") and (totalWidth > leftPos) ) then
				self:SetAnchorType(self:GetAnchorType(), (totalWidth - leftPos), 0);
			elseif ( (side == "right") and (rightPos + totalWidth) >  GetScreenWidth() ) then
				self:SetAnchorType(self:GetAnchorType(), -((rightPos + totalWidth) - GetScreenWidth()), 0);
			end
		end

		if ( secondaryItemShown ) then
			shoppingTooltip2:SetOwner(self, "ANCHOR_NONE");
			shoppingTooltip2:ClearAllPoints();
			if ( side and side == "left" ) then
				shoppingTooltip2:SetPoint("TOPRIGHT", anchor, "TOPLEFT", -8, 0);
			else
				shoppingTooltip2:SetPoint("TOPLEFT", anchor, "TOPRIGHT", 8, 0);
			end

			shoppingTooltip1:SetOwner(self, "ANCHOR_NONE");
			shoppingTooltip1:ClearAllPoints();

			if ( side and side == "left" ) then
				shoppingTooltip1:SetPoint("TOPRIGHT", shoppingTooltip2, "TOPLEFT", -8, 0);
			else
				shoppingTooltip1:SetPoint("TOPLEFT", shoppingTooltip2, "TOPRIGHT", 8, 0);
			end
		else
			shoppingTooltip1:SetOwner(self, "ANCHOR_NONE");
			shoppingTooltip1:ClearAllPoints();

			if ( side and side == "left" ) then
				shoppingTooltip1:SetPoint("TOPRIGHT", anchor, "TOPLEFT", -8, 0);
			else
				shoppingTooltip1:SetPoint("TOPLEFT", anchor, "TOPRIGHT", 8, 0);
			end

			shoppingTooltip2:Hide();
		end

		-- We have to call this again because :SetOwner clears the tooltip.
		shoppingTooltip1:SetCompareItem(shoppingTooltip2, self);
		shoppingTooltip1:Show();
end
hooksecurefunc('GameTooltip_ShowCompareItem', ShowCompareItem)

local GetMouseoverUnit = function()
	local _, unit = GameTooltip:GetUnit()
	if not unit or not UnitExists(unit) or UnitIsUnit(unit, 'mouseover') then
		return true
	else
		return false
	end
end

local GetTarget = function(unit)
	local _, class = UnitClass(unit)
	if UnitIsUnit(unit, 'player') then
		return ('|cffff0000%s|r'):format('-YOU-')
	elseif UnitIsPlayer(unit, 'player')then
		return ('|cff%s%s|r'):format(classColors[class], UnitName(unit))
	elseif UnitIsPlayer(unit, 'player') and UnitFactionGroup(unit) == 'Horde' then
		return ('|TInterface\\AddOns\\lyn\\assets\\horde.tga:12:12|t'..'|cff%s%s|r'):format(classColors[class], UnitName(unit))
	elseif UnitIsPlayer(unit, 'player') and UnitFactionGroup(unit) == 'Alliance' then
		return ('|TInterface\\AddOns\\lyn\\assets\\alliance.tga:12:12|t '..'|cff%s%s|r'):format(classColors[class], UnitName(unit))
	elseif UnitReaction(unit, 'player') then
		return ('|cff%s%s|r'):format(reactionColors[UnitReaction(unit, 'player')], UnitName(unit))
	else
		return ('|cffffffff%s|r'):format(UnitName(unit))
	end
end

local GetFormattedUnitLevel = function(unit)
	local diff = GetQuestDifficultyColor(UnitLevel(unit))
	if UnitLevel(unit) == -1 then
		return '|cffff0000??|r '
	elseif UnitLevel(unit) == 0 then
		return '? '
	else
	--classifications[UnitClassification(unit)] or ''
		return format('|cff%02x%02x%02x%s|r ', diff.r*255, diff.g*255, diff.b*255, UnitLevel(unit))
	end
end


-- CLASS
local GetFormattedUnitClass = function(unit)
	local _, class = UnitClass(unit)
	local color = CUSTOM_CLASS_COLORS and CUSTOM_CLASS_COLORS[class] or RAID_CLASS_COLORS[class]
	--local color = {r=1,g=1,b=1}
	if color then
		return format('|cff%02x%02x%02x%s|r', color.r*255, color.g*255, color.b*255, UnitClass(unit))
	end
end

-- classification
local GetFormattedClassification = function(unit)
	return classifications[UnitClassification(unit)] or ''
end

-- MOB TYPE
local GetFormattedUnitType = function(unit)
	local creaturetype = UnitCreatureType(unit)
	if creaturetype then return creaturetype
	else return ''
	end
end

local GetFormattedLevelString = function(unit, specIcon)
	if UnitIsPlayer(unit) then
		if not UnitRace(unit) then return nil end
		return GetFormattedUnitLevel(unit)..UnitRace(unit)..' '..GetFormattedUnitClass(unit)
	else
		return GetFormattedUnitLevel(unit)..GetFormattedClassification(unit)..GetFormattedUnitType(unit)
	end
end

local OnTooltipSetUnit = function(self, ...)
	local unit = select(2, self:GetUnit()) or (GetMouseFocus() and GetMouseFocus():GetAttribute('unit')) or (UnitExists('mouseover') and 'mouseover')
	if not unit or (unit and type(unit) ~= 'string') then return end
	if not UnitGUID(unit) then return end

	-- reaction colour
	local unitReaction = FACTION_BAR_COLORS[UnitReaction(unit, 'player')] and FACTION_BAR_COLORS[UnitReaction(unit, 'player')] or { r = .5, g = .5, b = .5 }

	-- raid icons
	local ricon = GetRaidTargetIndex(unit)
	if ricon then
		local text = GameTooltipTextLeft1:GetText()
		GameTooltipTextLeft1:SetText(('%s %s'):format(ICON_LIST[ricon]..'14|t', text))
	end
	
	-- hide/show titles
	local guild = GetGuildInfo(unit)
	local title = UnitPVPName(unit)
	local name = UnitName(unit)
	local relationship = UnitRealmRelationship(unit)
	--local faction = UnitFactionGroup(unit) == FACTION_HORDE and '|cff333333 / |r|cff852726H|r' or UnitFactionGroup(unit) == FACTION_ALLIANCE and '|cff333333 / |r|cff193FABA|r' or ''
	local faction = ''
	local flag = UnitIsAFK(unit) and '|cff333333 / |r|cff00ff00AFK|r' or UnitIsDND(unit) and '|cff333333 / |r|cff00ff00DND|r' or ''

	GameTooltipTextLeft1:SetText(name .. ((relationship == 2 or relationship == 3) and ' |cff444444#|r' or '') .. faction .. flag)
		
	if UnitIsPlayer(unit) then
		-- hp & name class colouring
		local _, class = UnitClass(unit)
		local colour = CUSTOM_CLASS_COLORS and CUSTOM_CLASS_COLORS[class] or RAID_CLASS_COLORS[class]
		local unitName, unitRealm = UnitName(unit)

		if not colour then
			colour = { r = 1, g = 1, b = 1 }
		end

		--GameTooltipStatusBar:SetStatusBarColor(colour.r, colour.g, colour.b)
		GameTooltipTextLeft1:SetTextColor(colour.r, colour.g, colour.b)
	else
		-- hp & name reaction colouring
		local reaction = UnitReaction(unit, 'player')
		if reaction then
			local colour = FACTION_BAR_COLORS[reaction] and FACTION_BAR_COLORS[reaction] or { r = .5, g = .5, b = .5 }
			if colour then
				 --GameTooltipStatusBar:SetStatusBarColor(colour.r, colour.g, colour.b)
				 GameTooltipTextLeft1:SetTextColor(colour.r, colour.g, colour.b)
			end
		 end
	end
	
	-- ghost/dead
	if UnitIsGhost(unit) then
		self:AppendText' |cffaaaaaa-GHOST-|r'
		GameTooltipTextLeft1:SetTextColor(.5, .5, .5)
	elseif UnitIsDead(unit) then
		self:AppendText' |cffaaaaaa-DEAD-|r'
		GameTooltipTextLeft1:SetTextColor(.5, .5, .5)
	end

	for i = 2, GameTooltip:NumLines() do
		local line = _G['GameTooltipTextLeft'..i]
		local text = line:GetText()
		if  line then
			local t = gsub(text, '(.+)', '%1')
			if t == '' or t == PVP_ENABLED
			or t == FACTION_ALLIANCE or t == FACTION_HORDE then
				line:SetText(nil)
			end
			
			if text and text:find('^'..TOOLTIP_UNIT_LEVEL:gsub('%%s', '.+')) then
                _G['GameTooltipTextLeft'..i]:SetText(GetFormattedLevelString(unit, specIcon))
            end
		end
	end
	
	-- guild
	if guild then
		local guildColor = UnitIsInMyGuild(unit) and '004af1' or 'ffffff'
		GameTooltip:AddLine('|cff'..guildColor..'<'..guild..'>|r')
	end

	-- target
	if UnitExists(unit..'target') then
		GameTooltip:AddLine('|cffffd100Target:|r ' .. GetTarget(unit..'target') or 'Unknown')
	end
		
	GameTooltip:SetBackdropColor(TOOLTIP_DEFAULT_BACKGROUND_COLOR.r, TOOLTIP_DEFAULT_BACKGROUND_COLOR.g, TOOLTIP_DEFAULT_BACKGROUND_COLOR.b, C.TOOLTIP_ALPHA)
end

local AddStatusBar = function(bar, tooltip)
	F:CreateStatusbarTexture(bar)
	
	bar:SetHeight(2)
	
	bar:ClearAllPoints()
	bar:SetPoint('LEFT', 1, 0)
	bar:SetPoint('RIGHT', -1, 0)
	bar:SetPoint('TOP', tooltip, 'BOTTOM', 0, 2)
	bar:SetOrientation('HORIZONTAL')
	bar:SetRotatesTexture(false)
	
	if not bar.Background then
		bar.Background = bar:CreateTexture(nil, 'BACKGROUND', nil, 7)
		F:CreateStatusbarTexture(bar.Background)
		bar.Background:SetAllPoints()
		bar.Background:SetVertexColor(.15, .15, .15)
	end
end

local UpdateStyle = function(self)
	local _, link = self:GetItem()
	F:CreateBorder(self, C.TOOLTIP_BORDER_INSET)
	self:SetBorderColor(unpack(C.BORDER_COLOR_DARK))
	if not self.Background then
		F:CreateBackground(self, C.TOOLTIP_ALPHA, C.TOOLTIP_BACKGROUND_INSET)
	end
	if self.Background then
		self.Background:SetBackdropColor(TOOLTIP_DEFAULT_BACKGROUND_COLOR.r, TOOLTIP_DEFAULT_BACKGROUND_COLOR.g, TOOLTIP_DEFAULT_BACKGROUND_COLOR.b, C.TOOLTIP_ALPHA)
	end
end

local SetDefaultAnchor = function(tooltip, parent)
	if not GetMouseoverUnit() then
		tooltip:SetOwner(parent, 'ANCHOR_CURSOR')
	else
		tooltip:ClearAllPoints()
		tooltip:SetPoint(unpack(C.TOOLTIP_POSITION))
		AddStatusBar(GameTooltipStatusBar, tooltip, false)
	end
end

local AddBackdrop = function(self)
	F:CreateBorder(self, C.TOOLTIP_BORDER_INSET)
	self:SetBorderColor(unpack(C.BORDER_COLOR_DARK))
	if not self.Background then
		F:CreateBackground(self, C.TOOLTIP_ALPHA, C.TOOLTIP_BACKGROUND_INSET)
	end
	if self.Background then
		self.Background:SetBackdropColor(TOOLTIP_DEFAULT_BACKGROUND_COLOR.r, TOOLTIP_DEFAULT_BACKGROUND_COLOR.g, TOOLTIP_DEFAULT_BACKGROUND_COLOR.b, C.TOOLTIP_ALPHA)
	end
end

local OnEvent = function(self, event)
	for _, v in next, tooltips do
		local tip = _G[v]

		tip:HookScript('OnShow', AddBackdrop)
		tip:HookScript('OnHide', AddBackdrop)
		tip:HookScript('OnTooltipCleared', AddBackdrop)
	end
	
	hooksecurefunc('GameTooltip_UpdateStyle', UpdateStyle)
	hooksecurefunc('GameTooltip_SetBackdropStyle', UpdateStyle)
	hooksecurefunc('GameTooltip_SetDefaultAnchor', SetDefaultAnchor)
end

local frame = CreateFrame('Frame')
frame:RegisterEvent('PLAYER_LOGIN')
frame:SetScript('OnEvent', OnEvent)

hooksecurefunc(GameTooltip, "Show", function(self)
	self:SetBackdropColor(TOOLTIP_DEFAULT_BACKGROUND_COLOR.r, TOOLTIP_DEFAULT_BACKGROUND_COLOR.g, TOOLTIP_DEFAULT_BACKGROUND_COLOR.b, C.TOOLTIP_ALPHA)
end)

GameTooltip:HookScript("OnUpdate", function(self, elapsed)
	self:SetBackdropColor(TOOLTIP_DEFAULT_BACKGROUND_COLOR.r, TOOLTIP_DEFAULT_BACKGROUND_COLOR.g, TOOLTIP_DEFAULT_BACKGROUND_COLOR.b, C.TOOLTIP_ALPHA)
end)

GameTooltip:HookScript('OnTooltipSetUnit', OnTooltipSetUnit)


hooksecurefunc("UIDropDownMenu_CreateFrames", function(level, index)
	for i = 1, UIDROPDOWNMENU_MAXLEVELS do
		local menu = _G["DropDownList"..i.."MenuBackdrop"]
		local backdrop = _G["DropDownList"..i.."Backdrop"]
		if not backdrop.skinned then
			
			F:CreateBorder(menu, C.TOOLTIP_BORDER_INSET)
			menu:SetBorderColor(unpack(C.BORDER_COLOR_DARK))
			
			if not menu.Background then
				F:CreateBackground(menu, C.TOOLTIP_ALPHA, C.TOOLTIP_BACKGROUND_INSET)
			end
			if menu.Background then			
				menu.Background:SetBackdropColor(TOOLTIP_DEFAULT_BACKGROUND_COLOR.r, TOOLTIP_DEFAULT_BACKGROUND_COLOR.g, TOOLTIP_DEFAULT_BACKGROUND_COLOR.b, C.TOOLTIP_ALPHA)
			end
			
			F:CreateBorder(backdrop, C.TOOLTIP_BORDER_INSET)
			backdrop:SetBorderColor(unpack(C.BORDER_COLOR_DARK))

			F:CreateBackground(backdrop, C.TOOLTIP_ALPHA, C.TOOLTIP_BACKGROUND_INSET)
			if backdrop.Background then
				backdrop.Background:SetBackdropColor(TOOLTIP_DEFAULT_BACKGROUND_COLOR.r, TOOLTIP_DEFAULT_BACKGROUND_COLOR.g, TOOLTIP_DEFAULT_BACKGROUND_COLOR.b, C.TOOLTIP_ALPHA)
			end

			backdrop.skinned = true
		end
	end
end)

local menus =    {
	ChatMenu,
	EmoteMenu,
	LanguageMenu,
	VoiceMacroMenu,
}
for i, menu in ipairs(menus) do
	F:CreateBorder(menu, C.TOOLTIP_BORDER_INSET)
	menu:SetBorderColor(unpack(C.BORDER_COLOR_DARK))
	
	if not menu.Background then
		F:CreateBackground(menu, C.TOOLTIP_ALPHA, C.TOOLTIP_BACKGROUND_INSET)
	end
	if menu.Background then			
		menu.Background:SetBackdropColor(TOOLTIP_DEFAULT_BACKGROUND_COLOR.r, TOOLTIP_DEFAULT_BACKGROUND_COLOR.g, TOOLTIP_DEFAULT_BACKGROUND_COLOR.b, C.TOOLTIP_ALPHA)
	end
end