local _, addon = ...
local cfg = addon.config
local lib = addon.lib
local E, F, C = unpack(_G['lyn'])

-- ----------------------------------------------------------------------------------------------------

local _, class = UnitClass('player')
local classColor = CUSTOM_CLASS_COLORS and CUSTOM_CLASS_COLORS[class] or RAID_CLASS_COLORS[class]

local CreateLayout = function(self, unit)
	self:SetScript('OnEnter', UnitFrame_OnEnter)
	self:SetScript('OnLeave', UnitFrame_OnLeave)
	self:RegisterForClicks('AnyUp')

	self:SetWidth(290)
	self:SetHeight(42)
	self:SetPoint('BOTTOM', UIParent, -170, 100)
		
	-- power bar
	local power = CreateFrame("StatusBar", nil, self)
	power:SetAllPoints(self)
	power:SetStatusBarTexture(cfg.statusbar.normal)
	power:SetStatusBarColor(.1, .1, .1)
	
	local pbg = power:CreateTexture(nil, "BACKGROUND", nil, 0)
	pbg:SetTexture(cfg.statusbar.normal)
	pbg:SetVertexColor(cfg.color.grey.r, cfg.color.grey.g, cfg.color.grey.b, 1)
	pbg:SetAllPoints(power)
	
	local border = CreateFrame('Frame', nil, self)
	border:SetAllPoints(power)
	border:SetSize(power:GetWidth(), power:GetHeight())
	border:SetFrameLevel(power:GetFrameLevel() + 3)
	lib.CreateFrameBorder(border, -2)
	border:SetBorderColor(unpack(C.BORDER_COLOR_DARK))
		
	-- power options
	power.colorPower = true
	power.altPowerColor = true
	power.frequentUpdates = true
	power.Smooth = true
	power.PostUpdate = lib.PostUpdatePower

	self.Power = power

	-- power prediction
	local predict = CreateFrame('StatusBar', nil, power)
	predict:SetPoint('TOP')
	predict:SetPoint('BOTTOM')
	predict:SetPoint('RIGHT', power:GetStatusBarTexture())
	predict:SetWidth(200)
	predict:SetFrameLevel(power:GetFrameLevel() + 1)
	predict:SetStatusBarTexture(cfg.statusbar.normal)
	--predict:SetStatusBarColor(192/255, 254/255, 190/255)
	predict:SetStatusBarColor(.55, .55, .55)
	predict:SetReverseFill(true)

	self.PowerPrediction = {
		mainBar = predict
	}

	-- health bar
	local health = CreateFrame("StatusBar", nil, self)
	health:SetStatusBarTexture(cfg.statusbar.gradient)
	health:SetStatusBarColor(.1, .1, .1)
	health:SetPoint('TOPLEFT', power, 3, -3)
	health:SetPoint('BOTTOMRIGHT', power, -3, 3)
	health:SetFrameLevel(power:GetFrameLevel() + 2)

	local hbg = health:CreateTexture(nil, "BACKGROUND", nil, 0)
	hbg:SetTexture(cfg.statusbar.gradient)
	hbg:SetVertexColor(cfg.color.grey.r, cfg.color.grey.g, cfg.color.grey.b, 1)
	hbg:SetAllPoints(health)
		
	local hborder = health:CreateTexture(nil, "BACKGROUND", nil, -2)
	hborder:SetColorTexture(0, 0, 0, 1)
	hborder:SetPoint('TOPLEFT', health, -1, 1)
	hborder:SetPoint('BOTTOMRIGHT', health, 1, -1)
	
	-- health options
	health.frequentUpdates = true
	health.colorReaction = true
	health.colorClass = true
	health.Smooth = true
	health.PostUpdate = lib.PostUpdateHealth

	self.Health = health

	-- we put everything on overlay to get it over the castbar
	local overlay = CreateFrame('Frame', nil, self)
	overlay:SetAllPoints(health)
	overlay:SetSize(health:GetWidth(), health:GetHeight())
	overlay:SetFrameLevel(power:GetFrameLevel() + 4)
	
	-- raid marker icon
	local marker = overlay:CreateTexture(nil, 'OVERLAY')
	marker:SetTexture(cfg.path .. 'raidicons')
	marker:SetSize(32, 32)
	marker:SetPoint('TOP', overlay, 'BOTTOM', 0, 16)
	self.RaidTargetIndicator = marker

	-- group leader
	local LeaderIndicator = overlay:CreateTexture(nil, "OVERLAY")
	LeaderIndicator:SetSize(13, 13)
	LeaderIndicator:SetTexture(cfg.path .. 'commander.tga')
	LeaderIndicator:SetPoint("BOTTOM", overlay, "TOP", 0, -3)
	LeaderIndicator:SetPoint("RIGHT", -6, 0)
	self.LeaderIndicator = LeaderIndicator

	-- pvp
	local pvp = self:CreateFontString(nil, "OVERLAY")
	pvp:SetPoint('TOP', self, 'BOTTOM', 0, -5)
	pvp:SetPoint('LEFT', 0, 0)
	--pvp:SetHeight(12)
	pvp:SetFont(cfg.font.big, 12, 'OUTLINE')
	pvp:SetShadowOffset(1, -1)
	pvp:SetShadowColor(0, 0, 0, 1)
	pvp:SetTextColor(1, 1, 1)
	self:Tag(pvp, '[lyn:pvp]')

	-- health percent
	local hp = overlay:CreateFontString(nil, "OVERLAY")
	hp:SetPoint('RIGHT', overlay, -7, 0)
	hp:SetShadowColor(0, 0, 0, 1)
	hp:SetTextColor(1, 1, 1)
	hp:SetShadowOffset(1, -1)
	hp:SetJustifyH('RIGHT')
	hp:SetFont(cfg.font.big, 30, 'OUTLINE')
	hp:SetTextColor(1, 1, 1)
	self:Tag(hp, '[lyn:hpp]')

	-- health absolute
	local hpv = overlay:CreateFontString(nil, "OVERLAY")
	hpv:SetPoint('RIGHT', hp, 'LEFT', -5, 1)
	hpv:SetJustifyH('RIGHT')
	hpv:SetFont(cfg.font.big, 12, 'OUTLINE')
	hpv:SetShadowOffset(1, -1)
	hpv:SetShadowColor(0, 0, 0, 1)
	hpv:SetTextColor(.8, .8, .8)
	self:Tag(hpv, '[lyn:hps]')

	-- power absolute
	local pp = overlay:CreateFontString(nil, "OVERLAY")
	pp:SetPoint('LEFT', overlay, 7, 1)
	pp:SetShadowColor(0, 0, 0, 1)
	pp:SetTextColor(1, 1, 1)
	pp:SetShadowOffset(1, -1)
	pp:SetJustifyH('LEFT')
	pp:SetFont(cfg.font.big, 30, 'OUTLINE')
	pp:SetTextColor(1, 1, 1)
	self:Tag(pp, '[lyn:power]')

	-- castbar
	lib.CreateCastbar(self, 'player')
		
	-- buffs
	local lines = 2
	if class == 'ROGUE' or class == 'DEATHKNIGHT' then
		lines = 3
	end
	local auras = lib.CreateAura(self, 12, lines, 32)
	auras:SetPoint('BOTTOM', self, 'TOP', 0, 9)
	auras:SetPoint('RIGHT', 0, 0)
	auras.initialAnchor = "BOTTOMRIGHT"
	auras.CustomFilter = lib.PlayerCustomFilter
	auras["growth-x"] = 'LEFT'
	auras["growth-y"] = 'UP'

	auras.PostUpdateIcon = lib.PostUpdateIcon
	self.Auras = auras

end

oUF:RegisterStyle('oUF_LynPlayer', CreateLayout)
oUF:SetActiveStyle('oUF_LynPlayer')
oUF:Spawn('player','oUF_LynPlayer')

-- if cfg.statusbar.normal then
	-- oUF:RegisterStyle('oUF_LynPlayer', CreateLayout)
	-- oUF:SetActiveStyle('oUF_LynPlayer')
	-- local player = oUF:Spawn('player','oUF_LynPlayer')
	-- player:Disable()
	-- RegisterStateDriver(player, "visibility", "[combat][@target,exists][group] show; hide")
-- end
