local _, addon = ...
local cfg = addon.config
local lib = addon.lib
local E, F, C = unpack(_G['lyn'])

-- ----------------------------------------------------------------------------------------------------

local CreateLayout = function(self, unit)
	self:SetScript('OnEnter', UnitFrame_OnEnter)
	self:SetScript('OnLeave', UnitFrame_OnLeave)
	self:RegisterForClicks('AnyUp')

	self:SetWidth(290)
	self:SetHeight(42)
	self:SetPoint('BOTTOM', UIParent, 170, 100)

	-- power bar
	local power = CreateFrame("StatusBar", nil, self)
	power:SetAllPoints(self)
	power:SetStatusBarTexture(cfg.statusbar.normal)
	power:SetStatusBarColor(.1, .1, .1)
	
	local pbg = power:CreateTexture(nil, "BACKGROUND", nil, 0)
	pbg:SetTexture(cfg.statusbar.normal)
	pbg:SetVertexColor(cfg.color.grey.r, cfg.color.grey.g, cfg.color.grey.b, 1)
	pbg:SetAllPoints(power)
	
	local border = CreateFrame('Frame', nil, self)
	border:SetAllPoints(power)
	border:SetSize(power:GetWidth(), power:GetHeight())
	border:SetFrameLevel(power:GetFrameLevel() + 3)
	lib.CreateFrameBorder(border, -2)
	border:SetBorderColor(unpack(C.BORDER_COLOR_DARK))

	-- power options
	power.colorPower = true
	power.altPowerColor = true
	power.frequentUpdates = true
	power.Smooth = true
	power.PostUpdate = lib.PostUpdatePower

	self.Power = power

	-- health bar
	local health = CreateFrame("StatusBar", nil, self)
	health:SetStatusBarTexture(cfg.statusbar.gradient)
	health:SetStatusBarColor(.1, .1, .1)
	health:SetPoint('TOPLEFT', power, 3, -3)
	health:SetPoint('BOTTOMRIGHT', power, -3, 3)
	health:SetFrameLevel(power:GetFrameLevel() + 1)

	local hbg = health:CreateTexture(nil, "BACKGROUND", nil, 0)
	hbg:SetTexture(cfg.statusbar.gradient)
	hbg:SetVertexColor(cfg.color.grey.r, cfg.color.grey.g, cfg.color.grey.b, 1)
	hbg:SetAllPoints(health)
		
	local hborder = health:CreateTexture(nil, "BACKGROUND", nil, -2)
	hborder:SetColorTexture(0, 0, 0, 1)
	hborder:SetPoint('TOPLEFT', health, -1, 1)
	hborder:SetPoint('BOTTOMRIGHT', health, 1, -1)

	-- health options
	health.frequentUpdates = true
	health.colorTapping = true
	health.colorClass = true
	health.Smooth = true
	health.colorReaction = true
	health.PostUpdate = lib.PostUpdateHealth

	self.Health = health

	-- we put everything on overlay to get it over the castbar
	local overlay = CreateFrame('Frame', nil, self)
	overlay:SetAllPoints(health)
	overlay:SetSize(health:GetWidth(), health:GetHeight())
	overlay:SetFrameLevel(power:GetFrameLevel() + 4)
	
	-- name
	local name = overlay:CreateFontString(nil, "OVERLAY")
	name:SetJustifyH('LEFT')
	name:SetShadowColor(0, 0, 0, 1)
	name:SetTextColor(1, 1, 1)
	name:SetShadowOffset(1,-1)
	name:SetFont(cfg.font.big, 18, 'OUTLINE')
	name:SetPoint("LEFT", overlay, 7, 0)
	name:SetWidth(320)

	self.Name = name
	self:Tag(self.Name, '[lyn:targetname]')
	
	-- raid marker icon
	local marker = overlay:CreateTexture(nil, 'OVERLAY')
	marker:SetTexture(cfg.path .. 'raidicons')
	marker:SetSize(32, 32)
	marker:SetPoint('TOP', overlay, 'BOTTOM', 0, 16)
	self.RaidTargetIndicator = marker

	-- classification (boss, elite, rare, ...)
	local classification = overlay:CreateFontString(nil, "OVERLAY")
	classification:SetPoint('BOTTOM', self, 'TOP', 0, -5)
	classification:SetPoint('RIGHT', -6, 0)
	classification:SetJustifyH('RIGHT')
	classification:SetFont(cfg.font.big, 12, 'OUTLINE')
	classification:SetShadowOffset(1, -1)
	classification:SetShadowColor(0, 0, 0, 1)
	classification:SetTextColor(1, 1, 1)
	classification:SetHeight(10)
	self:Tag(classification, '[lyn:classification]')

	-- pvp
	local PvPIndicator = overlay:CreateFontString(nil, "OVERLAY")
	PvPIndicator:SetPoint('BOTTOM', self, 'TOP', 0, -5)
	PvPIndicator:SetPoint('LEFT', 6, 0)
	PvPIndicator:SetJustifyH('LEFT')
	PvPIndicator:SetHeight(10)
	PvPIndicator:SetFont(cfg.font.big, 12, 'OUTLINE')
	PvPIndicator:SetShadowOffset(1, -1)
	PvPIndicator:SetShadowColor(0, 0, 0, 1)
	PvPIndicator:SetTextColor(1, 1, 1)
	self:Tag(PvPIndicator, '[lyn:pvp]')

	-- health percent
	local hp = overlay:CreateFontString(nil, "OVERLAY")
	hp:SetFont(cfg.font.big, 30, 'OUTLINE')
	hp:SetShadowOffset(1,-1)
	hp:ClearAllPoints()
	hp:SetPoint("RIGHT", overlay, -7, 0)
	self:Tag(hp, '[lyn:hpp]')
	health.value = hp

	-- buffs
	local buffs = lib.CreateAura(self, 13, 1, 22)
	--buffs:SetPoint("TOP", overlay, "BOTTOM", 0, UnitIsPlayer(unit) and -9 or -12)
	buffs:SetPoint("TOP", overlay, "BOTTOM", 0, -10)
	buffs:SetPoint('LEFT', 0, 0)
	buffs.initialAnchor = "TOPLEFT"
	buffs["growth-x"] = 'RIGHT'
	buffs["growth-y"] = 'DOWN'

	buffs.PostUpdateIcon = lib.PostUpdateIcon
	self.Buffs = buffs

	-- debuffs
	local debuffs = lib.CreateAura(self, 20, 4, 32)
	debuffs:SetPoint('BOTTOM', overlay, 'TOP', 0, 9)
	debuffs:SetPoint('LEFT', 0, 0)
	debuffs.showDebuffType = true
	debuffs.onlyShowPlayer = cfg.onlyPlayerDebuffs
	debuffs.showStealableBuffs = true
	debuffs.initialAnchor = 'BOTTOMLEFT'
	debuffs["growth-x"] = 'RIGHT'

	debuffs.PostUpdateIcon = lib.PostUpdateIcon
	self.Debuffs = debuffs

	-- castbar
	lib.CreateCastbar(self, 'target')
end

oUF:RegisterStyle('oUF_LynTarget', CreateLayout)
oUF:SetActiveStyle('oUF_LynTarget')
oUF:Spawn('target','oUF_LynTarget')
